Re: [PATCH v3 03/31] Add parse_pathspec() that converts cmdline args to struct pathspec

2013-01-13 Thread Duy Nguyen
On Mon, Jan 14, 2013 at 7:05 AM, Martin von Zweigbergk
 wrote:
> On Sun, Jan 13, 2013 at 4:35 AM, Nguyễn Thái Ngọc Duy  
> wrote:
>> +static void parse_pathspec(struct pathspec *pathspec,
>> +  unsigned magic_mask, unsigned flags,
>> +  const char *prefix, const char **argv)
>> +{
>> +   struct pathspec_item *item;
>> +   const char *entry = *argv;
>> ...
>> +   for (i = 0; i < n; i++) {
>> +   const char *arg = argv[i];
>
> Nit: "*argv" was assigned to "entry" above. Reassign that variable instead?

Yeah. Thanks for catching.
-- 
Duy
--
To unsubscribe from this list: send the line "unsubscribe git" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html


Re: [PATCH v3 03/31] Add parse_pathspec() that converts cmdline args to struct pathspec

2013-01-13 Thread Martin von Zweigbergk
On Sun, Jan 13, 2013 at 4:35 AM, Nguyễn Thái Ngọc Duy  wrote:
> +static void parse_pathspec(struct pathspec *pathspec,
> +  unsigned magic_mask, unsigned flags,
> +  const char *prefix, const char **argv)
> +{
> +   struct pathspec_item *item;
> +   const char *entry = *argv;
> ...
> +   for (i = 0; i < n; i++) {
> +   const char *arg = argv[i];

Nit: "*argv" was assigned to "entry" above. Reassign that variable instead?
--
To unsubscribe from this list: send the line "unsubscribe git" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html


[PATCH v3 03/31] Add parse_pathspec() that converts cmdline args to struct pathspec

2013-01-13 Thread Nguyễn Thái Ngọc Duy
Currently to fill a struct pathspec, we do:

   const char **paths;
   paths = get_pathspec(prefix, argv);
   ...
   init_pathspec(&pathspec, paths);

"paths" can only carry bare strings, which loses information from
command line arguments such as pathspec magic or the prefix part's
length for each argument.

parse_pathspec() is introduced to combine the two calls into one. The
plan is gradually replace all get_pathspec() and init_pathspec() with
parse_pathspec(). get_pathspec() now becomes a thin wrapper of
parse_pathspec().

parse_pathspec() allows the caller to reject the pathspec magics that
it does not support. When a new pathspec magic is introduced, we can
enable it per command after making sure that all underlying code has no
problem with the new magic.

"flags" parameter is currently unused. But it would allow callers to
pass certain instructions to parse_pathspec, for example forcing
literal pathspec when no magic is used.

With the introduction of parse_pathspec, there are now two functions
that can initialize struct pathspec: init_pathspec and
parse_pathspec. Any semantic changes in struct pathspec must be
reflected in both functions. init_pathspec() will be phased out in
favor of parse_pathspec().

Signed-off-by: Nguyễn Thái Ngọc Duy 
---
 cache.h |   2 ++
 dir.c   |   4 +--
 dir.h   |   2 ++
 setup.c | 108 +---
 4 files changed, 90 insertions(+), 26 deletions(-)

diff --git a/cache.h b/cache.h
index 72675a1..759c62a 100644
--- a/cache.h
+++ b/cache.h
@@ -481,9 +481,11 @@ struct pathspec {
int nr;
unsigned int has_wildcard:1;
unsigned int recursive:1;
+   unsigned magic;
int max_depth;
struct pathspec_item {
const char *match;
+   unsigned magic;
int len;
int nowildcard_len;
int flags;
diff --git a/dir.c b/dir.c
index 12a76d7..8454c13 100644
--- a/dir.c
+++ b/dir.c
@@ -323,7 +323,7 @@ int match_pathspec_depth(const struct pathspec *ps,
 /*
  * Return the length of the "simple" part of a path match limiter.
  */
-static int simple_length(const char *match)
+int simple_length(const char *match)
 {
int len = -1;
 
@@ -335,7 +335,7 @@ static int simple_length(const char *match)
}
 }
 
-static int no_wildcard(const char *string)
+int no_wildcard(const char *string)
 {
return string[simple_length(string)] == '\0';
 }
diff --git a/dir.h b/dir.h
index ae1bc46..0cf5ccf 100644
--- a/dir.h
+++ b/dir.h
@@ -88,6 +88,8 @@ struct dir_struct {
 #define MATCHED_RECURSIVELY 1
 #define MATCHED_FNMATCH 2
 #define MATCHED_EXACTLY 3
+extern int simple_length(const char *match);
+extern int no_wildcard(const char *string);
 extern char *common_prefix(const char **pathspec);
 extern int match_pathspec(const char **pathspec, const char *name, int 
namelen, int prefix, char *seen);
 extern int match_pathspec_depth(const struct pathspec *pathspec,
diff --git a/setup.c b/setup.c
index f108c4b..92adefc 100644
--- a/setup.c
+++ b/setup.c
@@ -174,7 +174,7 @@ static struct pathspec_magic {
 
 /*
  * Take an element of a pathspec and check for magic signatures.
  * Append the result to the prefix.
  *
  * For now, we only parse the syntax and throw out anything other than
  * "top" magic.
@@ -185,10 +185,14 @@ static struct pathspec_magic {
  * the prefix part must always match literally, and a single stupid
  * string cannot express such a case.
  */
-static const char *prefix_pathspec(const char *prefix, int prefixlen, const 
char *elt)
+static unsigned prefix_pathspec(struct pathspec_item *item,
+   const char **raw, unsigned flags,
+   const char *prefix, int prefixlen,
+   const char *elt)
 {
unsigned magic = 0;
const char *copyfrom = elt;
+   char *match;
int i;
 
if (elt[0] != ':') {
@@ -241,39 +245,95 @@ static const char *prefix_pathspec(const char *prefix, 
int prefixlen, const char
}
 
if (magic & PATHSPEC_FROMTOP)
-   return xstrdup(copyfrom);
+   match = xstrdup(copyfrom);
else
-   return prefix_path(prefix, prefixlen, copyfrom);
+   match = prefix_path(prefix, prefixlen, copyfrom);
+   *raw = item->match = match;
+   item->len = strlen(item->match);
+   item->flags = 0;
+   if (limit_pathspec_to_literal())
+   item->nowildcard_len = item->len;
+   else
+   item->nowildcard_len = simple_length(item->match);
+   if (item->nowildcard_len < item->len &&
+   item->match[item->nowildcard_len] == '*' &&
+   no_wildcard(item->match + item->nowildcard_len + 1))
+   item->flags |= PATHSPEC_ONESTAR;
+   return magic;
 }
 
-const char **get_pathspec(const char *prefix, const char **pathspec)
+sta
@@ -174,7 +174,7 @@ static struct pathspec_magic {
 /*
  * Take an element of a pathspec and check for magic signatures.
- * Append the result to the prefix.
+ * Append the result to the prefix. Return the magic bitmap.
  *
  * For now, we only parse the syntax and throw out anything other than